@@ -12,9 +12,10 @@ import sys
 
 import django
 from django.conf import settings
-from django.db import models
 from django.http import HttpResponse
 
+from dirtyfields.dirtyfields import reset_state
+
 from django_declarative_apis.machinery.filtering import apply_filters_to_object
 from django_declarative_apis.models import BaseConsumer
 from django_declarative_apis.resources.utils import HttpStatusCode
@@ -37,7 +38,7 @@ from .attributes import (
     ResourceField,
 )
 
-# these imports are unusued in this file but may be used in other projects
+# these imports are unused in this file but may be used in other projects
 # that use `machinery` as an interface
 from .attributes import TypedEndpointAttributeMixin, RequestFieldGroup  # noqa
 from .utils import locate_object, rate_limit_exceeded
@@ -53,15 +54,19 @@ logger = logging.getLogger(__name__)
 
 
 class EndpointResourceAttribute(EndpointAttribute):
-    """It is used as a decorator on a resource function. It specifies the attributes of that resource.
+    """Used as a decorator on a resource function. Specifies the attributes of that
+    resource.
 
-    :param type: Specifies the model type. It is used only for documentation generation purposes.
+    :param type: Specifies the model type. It is used only for documentation generation
+        purposes.
     :type type: required
 
-    :param filter: Defines the class filters. Overrides the default filters. Defaults to :code:`None`.
+    :param filter: Defines the class filters. Overrides the default filters. Defaults to
+        :code:`None`.
     :type filter: optional
 
-    :param returns_list:  It is used for documentation generation purposes. Defaults to :code:`False`
+    :param returns_list:  It is used for documentation generation purposes. Defaults to
+        :code:`False`
     :type returns_list: optional
 
     **Example**
@@ -70,7 +75,10 @@ class EndpointResourceAttribute(EndpointAttribute):
 
         from django_declarative_apis.machinery import endpoint_resource
 
-        class TodoSingleTaskDefinition(TodoResourceMixin, machinery.ResourceEndpointDefinition):
+        class TodoSingleTaskDefinition(
+            TodoResourceMixin,
+            machinery.ResourceEndpointDefinition
+        ):
             resource_id = url_field(name='id')  # grabs the id from url
 
             @endpoint_resource(type=Todo)
@@ -78,8 +86,8 @@ class EndpointResourceAttribute(EndpointAttribute):
                 return Todo.objects.get(id=self.resource_id)
     """
 
-    def __init__(self, type, filter=None, returns_list=False, **kwargs):
-        super(EndpointResourceAttribute, self).__init__(**kwargs)
+    def __init__(self, type=None, filter=None, returns_list=False, **kwargs):
+        super().__init__(**kwargs)
         self.type = type
         self.filter = filter
         self.func = None
@@ -94,10 +102,12 @@ class EndpointResourceAttribute(EndpointAttribute):
             return self
         try:
             value = self.func(owner_instance)
-        except django.core.exceptions.ObjectDoesNotExist:
-            raise errors.ClientErrorNotFound(
-                "{0} instance not found".format(self.type.__name__)
-            )
+        except django.core.exceptions.ObjectDoesNotExist as e:  # noqa: F841
+            try:
+                message = f"{self.type.__name__} instance not found"
+            except AttributeError as e:  # noqa: F841
+                message = "Resource instance not found"
+            raise errors.ClientErrorNotFound(message)
 
         if value.__class__ == dict:
             return value
@@ -109,12 +119,15 @@ class EndpointResourceAttribute(EndpointAttribute):
 
 
 class EndpointResponseAttribute(EndpointAttribute):
-    """It is used as a decorator on a response function. It specifies the attributes of the response.
+    """Used as a decorator on a response function. Specifies the attributes of the
+    response.
 
-    :param type: Specifies the response type, which can be dictionary, list, or model type. It is used only for documentation generation purposes.
+    :param type: Specifies the response type, which can be dictionary, list, or
+        type. It is used only for documentation generation purposes.
     :type type: required
 
-    :param filter: Defines the class filters. Overrides the default filters. Defaults to :code:`None`.
+    :param filter: Defines the class filters. Overrides the default filters.
+        Defaults to :code:`None`.
     :type filter: optional
 
     **Example**
@@ -129,7 +142,7 @@ class EndpointResponseAttribute(EndpointAttribute):
     """
 
     def __init__(self, type, filter=None, **kwargs):
-        super(EndpointResponseAttribute, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.type = type
         self.filter = filter
         self.func = None
@@ -152,8 +165,9 @@ class EndpointDefinitionMeta(abc.ABCMeta, metaclass=abc.ABCMeta):
     def __init__(cls, class_name, bases=None, dict=None):
         super(EndpointDefinitionMeta, cls).__init__(class_name, bases, dict)
 
-        # This metaclass sets EndpointAttributeDiscriptor's names if they haven't otherwise been set
-        # This will walk parent classes as well so that attributes can be defined through inheritance
+        # This metaclass sets EndpointAttributeDescriptor's names if they haven't
+        # otherwise been set. This will walk parent classes as well so that attributes
+        # can be defined through inheritance
         ancestor_attribs = (ancestor.__dict__.items() for ancestor in cls.mro())
         for name, attribute in itertools.chain(dict.items(), *ancestor_attribs):
             try:
@@ -163,15 +177,45 @@ class EndpointDefinitionMeta(abc.ABCMeta, metaclass=abc.ABCMeta):
                 pass
 
 
-class EndpointBinder(object):
-    class BoundEndpointManager(object):
+def current_dirty_dict(resource):
+    """Get the `current` (in-memory) values for fields that have not yet been written to the database."""
+    new_data = resource.get_dirty_fields(check_relationship=True, verbose=True)
+    field_name_to_att_name = {f.name: f.attname for f in resource._meta.concrete_fields}
+    return {
+        field_name_to_att_name[key]: values["current"]
+        for key, values in new_data.items()
+    }
+
+
+def update_dirty(resource):
+    """Write dirty fields to the database."""
+    dirty_dict = current_dirty_dict(resource)
+    resource_next, created = type(resource).objects.update_or_create(
+        pk=resource.pk, defaults=dirty_dict
+    )
+
+    # update fields in memory that changed on save to the database
+    field_name_to_att_name = {f.name: f.attname for f in resource._meta.concrete_fields}
+    for k, v in resource_next._as_dict(check_relationship=True).items():
+        att_key = field_name_to_att_name[k]
+        if getattr(resource, att_key, None) != v:
+            setattr(resource, att_key, v)
+    resource._state.adding = False
+    resource._state.db = resource_next._state.db
+    resource._state.fields_cache = {}
+    reset_state(type(resource), resource)
+
+
+class EndpointBinder:
+    class BoundEndpointManager:
         def __init__(self, manager, bound_endpoint):
             self.manager = manager
             self.bound_endpoint = bound_endpoint
             self.binding_exc_info = None
             self.validation_exc_info = None
 
-        def get_response(self):
+        # TODO: make this method less complex and remove the `noqa`
+        def get_response(self):  # noqa: C901
             error = self.binding_exc_info or self.validation_exc_info
             if error:
                 exc_type, exc_value, exc_traceback = error
@@ -186,7 +230,7 @@ class EndpointBinder(object):
 
             if hasattr(resource, "is_dirty"):
                 if resource and resource.is_dirty(check_relationship=True):
-                    resource.save()
+                    update_dirty(resource)
 
             endpoint_tasks = sorted(
                 self.manager.endpoint_tasks, key=lambda t: t.priority
@@ -202,13 +246,18 @@ class EndpointBinder(object):
                     immediate_task.run(self.bound_endpoint)
 
             except errors.ClientError as ce:
-                if ce.save_changes and resource and resource.is_dirty():
-                    resource.save()
+                if (
+                    ce.save_changes
+                    and resource
+                    and hasattr(resource, "is_dirty")
+                    and resource.is_dirty()
+                ):
+                    update_dirty(resource)
                 raise
 
             if hasattr(resource, "is_dirty"):
                 if resource and resource.is_dirty(check_relationship=True):
-                    resource.save()
+                    update_dirty(resource)
 
             for deferred_task in deferred_tasks:
                 deferred_task.run(self.bound_endpoint)
@@ -238,7 +287,7 @@ class EndpointBinder(object):
                 )
 
     def __init__(self, endpoint_definition):
-        super(EndpointBinder, self).__init__()
+        super().__init__()
         self.endpoint_definition = endpoint_definition
         self.endpoint_attributes = endpoint_definition.get_endpoint_attributes()
         self.request_properties = endpoint_definition.get_request_properties()
@@ -271,8 +320,8 @@ class EndpointBinder(object):
                 }
             )
 
-        # Bind the request object within the instance (this allows RequestProperties to access the request
-        # without the endpoint definition having direct access to it)
+        # Bind the request object within the instance (this allows RequestProperties to
+        # access the request without the endpoint definition having direct access to it)
         RequestProperty.bind_request_to_instance(endpoint, request)
 
         bound_endpoint_manager = EndpointBinder.BoundEndpointManager(manager, endpoint)
@@ -291,7 +340,8 @@ class EndpointBinder(object):
         return bound_endpoint_manager
 
     def _bind_endpoint(self, endpoint):
-        # Access all request properties (this validates a request using the definition and caches the values)
+        # Access all request properties (this validates a request using the definition
+        # and caches the values)
         extra_error_message = ""
         missing_required_properties = []
         invalid_value_properties = []
@@ -345,9 +395,9 @@ class EndpointBinder(object):
             raise errors.ClientErrorRequestThrottled()
 
 
-class _EndpointRequestLifecycleManager(object):
+class _EndpointRequestLifecycleManager:
     def __init__(self, endpoint_definition):
-        super(_EndpointRequestLifecycleManager, self).__init__()
+        super().__init__()
         self.endpoint_definition = endpoint_definition
         self.binder = EndpointBinder(endpoint_definition)
         self.endpoint_tasks = endpoint_definition.get_tasks()
@@ -363,9 +413,9 @@ class _EndpointRequestLifecycleManager(object):
         return self.endpoint_definition.__name__
 
 
-class BehavioralEndpointDefinitionRouter(object):
+class BehavioralEndpointDefinitionRouter:
     def __init__(self, *endpoint_definitions):
-        super(BehavioralEndpointDefinitionRouter, self).__init__()
+        super().__init__()
         self.endpoint_definitions = endpoint_definitions
         self.endpoint_managers = [
             _EndpointRequestLifecycleManager(endpoint)
@@ -415,24 +465,24 @@ class EndpointDefinitionMixin(metaclass=EndpointDefinitionMeta):
 
 
 class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
-    """It is the base class for implementing Endpoints. At the very least a developer needs to inherit from
-    :code:`BaseEndpointDefinition` class.
-    This is how the EndpointBinder will know how to communicate with the endpoint and query its fields.
+    """The base class for implementing Endpoints. At the very least a developer needs to
+    inherit from :code:`BaseEndpointDefinition` class. This is how the EndpointBinder
+    will know how to communicate with the endpoint and query its fields.
     """
 
     @abc.abstractmethod
     def is_authorized(self):
-        """The authentication layer of DDA that is tied to the resource adapter is only responsible
-        for validating the requester.
-        We still need to determine whether the requester is authorized to perform certain actions,
-        which is the reason behind implementation of :code:`is_authorized`.
+        """The authentication layer of DDA that is tied to the resource adapter is only
+        responsible for validating the requester. We still need to determine whether the
+        requester is authorized to perform certain actions, which is the reason behind
+        implementation of :code:`is_authorized`.
 
-        :code:`is_authorized` performs authorization check on the request to decide whether or not the user should have
-        access to the resource, and returns a boolean value.
+        :code:`is_authorized` performs an authorization check on the request to decide
+        whether or not the user should have access to the resource and returns a boolean
+        value.
 
-
-        :code:`is_authorized` implementation should be overridden by the endpoint definition
-        inheriting from :code:`BaseEndpointDefinition`
+        :code:`is_authorized` implementation should be overridden by the endpoint
+        definition inheriting from :code:`BaseEndpointDefinition`
 
         **Default Value |** :code:`False`
 
@@ -443,29 +493,30 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
 
             from django_declarative_apis import machinery
 
-            SampleEndpointDefinition(machinery.BaseEndpointDefinition):
+            class SampleEndpointDefinition(machinery.BaseEndpointDefinition):
                 def is_authorized(self):
                     return True
-            Authorization check. Should be overridden by endpoint definition implementations.
         """
         return False
 
     def is_permitted(self):
-        """Similar to code:`is_authorized`, it checks whether a user has the permission to access the resource. Returns a boolean value.
+        """Similar to :code:`is_authorized`, it checks whether a user has the permission
+        to access the resource. Returns a boolean value.
 
         **Default Value |** :code:`True`
         """
         return True
 
     def is_valid(self):
-        """It can be used for scenarios where a request binds correctly, however, there are combination of parameters
-        that would make the request invalid. Returns a boolean value.
+        """Used in scenarios where a request binds correctly but there are combination
+        of parameters that would make the request invalid. Returns a boolean value.
 
-        For example, if the valid value for a field is from 1 to 10, this cannot be expressed through :code:`field`.
-        However, we can use the :code:`is_valid` to express it.
+        For example, if the valid value for a field is from 1 to 10, this cannot be
+        expressed through :code:`field`. However, we can use the :code:`is_valid` to
+        express it.
 
-        An alternative to :code:`is_valid` would be to use the :code:`@field` as a decorator on a function
-        and express this restriction there.
+        An alternative to :code:`is_valid` would be to use the :code:`@field` as a
+        decorator on a function and express this restriction there.
 
         **Default Value |** :code:`True`
 
@@ -475,7 +526,7 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
 
             from django_declarative_apis import machinery
 
-            SampleEndpointDefinition(machinery.BaseEndpointDefinition):
+            class SampleEndpointDefinition(machinery.BaseEndpointDefinition):
                 valid_int = field(required=True, type=int)
 
                 def is_authorized(self):
@@ -497,7 +548,8 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
         return None
 
     def rate_limit_period(self):
-        """Specifies and returns the number of seconds to enforce between requests with the same :code:`rate_limit_key`.
+        """Specifies and returns the number of seconds to enforce between requests with
+        the same :code:`rate_limit_key`.
 
         **Default Value |** 1
         """
@@ -523,12 +575,15 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
     @property
     @abc.abstractmethod
     def resource(self):
-        """Instance of a resource should either be a dictionary or instance of a Django Model or QuerySet.
+        """Instance of a resource should either be a dictionary or instance of a Django
+        Model or QuerySet.
 
-        This property **must** be implemented by all endpoint definitions. If not implemented, it will raise a NotImplementedError.
+        This property **must** be implemented by all endpoint definitions. If not
+        implemented, it will raise a NotImplementedError.
 
         .. note::
-            **Important**: The DDA framework will by default return self.resource as the response, unless response is overridden to return something else.
+            **Important**: The DDA framework will by default return self.resource as the
+            response, unless response is overridden to return something else.
 
         **Example**
 
@@ -563,7 +618,10 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
 
             from django_declarative_apis import machinery
 
-            class TodoUpdateSingleTaskDefinition(TodoResourceMixin, machinery.ResourceEndpointDefinition):
+            class TodoUpdateSingleTaskDefinition(
+                TodoResourceMixin,
+                machinery.ResourceEndpointDefinition,
+            ):
                 task = field(required=True, type=str)
                 priority = field(required=True, type=str)
                 completion_status = field(type=bool, default=False)
@@ -578,7 +636,8 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
                     task.save()
                     return task
 
-        Using :code:`get_endpoint_attributes` to find all the attributes of this endpoint and print it.
+        Using :code:`get_endpoint_attributes` to find all the attributes of this
+        endpoint and print it:
 
         .. code-block:: python
 
@@ -752,7 +811,8 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
 
     @classmethod
     def documentation(cls):
-        """Returns a dictionary containing the class name and endpoint fields that can be used for documentation purposes.
+        """Returns a dictionary containing the class name and endpoint fields that can
+        be used for documentation purposes.
 
         **Example**::
 
@@ -760,7 +820,8 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
             'fields': [{'name': 'request'},
                        {'name': 'task', 'type': <class 'str'>, 'multivalued': False},
                        {'name': 'priority', 'type': <class 'str'>, 'multivalued': False},
-                       {'name': 'completion_status', 'type': <class 'bool'>, 'multivalued': False, 'default_value': False}
+                       {'name': 'completion_status', 'type': <class 'bool'>,
+                        'multivalued': False, 'default_value': False}
                        ],
             'consumer_type': 'unknown'}
         """
@@ -780,8 +841,8 @@ class BaseEndpointDefinition(metaclass=EndpointDefinitionMeta):
 
 
 class EndpointDefinition(BaseEndpointDefinition):
-    """This base class can be used for implementing endpoints that
-    are not tied to a model. It also implements a basic consumer-based authentication.
+    """This base class can be used for implementing endpoints that are not tied to a
+    model. It also implements a basic consumer-based authentication.
     """
 
     request = RawRequestObjectProperty()
@@ -791,7 +852,8 @@ class EndpointDefinition(BaseEndpointDefinition):
     """Defines the consumer type with the default privileges of read and write.
 
     .. note::
-        If you do not want to define a consumer for your api, set :code:`consumer` and :code:`_consumer_type` to :code:`None`.
+        If you do not want to define a consumer for your api, set
+        :code:`consumer` and :code:`_consumer_type` to :code:`None`.
     """
 
     is_read_only = False
@@ -831,7 +893,9 @@ class EndpointDefinition(BaseEndpointDefinition):
 
     @classmethod
     def get_consumer_type(cls):
-        """Returns consumer type. If consumer is set to :code:`None` it will return unknown."""
+        """Returns consumer type. If consumer is set to :code:`None` it will return
+        unknown.
+        """
         consumer_attribute = cls.get_consumer_attributes()
         if len(consumer_attribute) == 1:
             consumer_attribute = consumer_attribute[0]
@@ -841,13 +905,15 @@ class EndpointDefinition(BaseEndpointDefinition):
 
     @classmethod
     def documentation(cls):
-        """Returns a dictionary containing class name, fields, and consumer type. Used for documentation purposes."""
+        """Returns a dictionary containing class name, fields, and consumer type. Used
+        for documentation purposes.
+        """
         docs = super().documentation()
         docs["consumer_type"] = cls.get_consumer_type()
         return docs
 
 
-class ResourceCreationMixin(object):
+class ResourceCreationMixin:
     @property
     def http_status(self):
         return http.client.CREATED
@@ -861,7 +927,8 @@ class ResourceEndpointDefinition(EndpointDefinition):
     """
 
     consumer = RequestAttribute()
-    """Initialize consumer using :code:`request_attribute()`. It can also be set to :code:`None`.
+    """Initialize consumer using :code:`request_attribute()`. It can also be set to
+    :code:`None`.
     """
 
     resource_id = RequestUrlField(
@@ -870,30 +937,34 @@ class ResourceEndpointDefinition(EndpointDefinition):
     """ The ID of the resource being fetched from the URL or being updated.
     """
     resource_model = None
-    """ The model to attach to the resource endpoint definition. 
+    """ The model to attach to the resource endpoint definition.
     It must extend or implement the Django ORM model interface as required.
     """
 
     def __init__(self, *args, **kwargs):
-        super(ResourceEndpointDefinition, self).__init__()
+        super().__init__(*args, **kwargs)
         self._cached_resource = None
 
-    @property
+    @EndpointResourceAttribute()
     def resource(self):
-        """Queries the object manager of `self.resource_model` for the given id (`self.resource_id`)."""
+        """Queries the object manager of `self.resource_model` for the given id
+        (`self.resource_id`).
+        """
         if not self._cached_resource:
             self._cached_resource = self.resource_model.objects.get(id=self.resource_id)
         return self._cached_resource
 
 
 class ResourceUpdateEndpointDefinition(ResourceEndpointDefinition):
-    """It handles the changes to the resource that happened from the
-    request, and saves the resource. It can be used for :code:`POST` and :code:`PUT`.
+    """Handles the changes to the resource that happened from the request, and saves the
+    resource. It can be used for :code:`POST` and :code:`PUT`.
     """
 
     @EndpointTask(priority=-100)
     def mutate(self):
-        """Modifies values of the resource fields by mapping the values of endpoint attributes to the resource."""
+        """Modifies values of the resource fields by mapping the values of endpoint
+        attributes to the resource.
+        """
         resource = self.resource
         for resource_field in self.get_resource_fields():
             field_value = getattr(self, resource_field.name)
@@ -902,8 +973,9 @@ class ResourceUpdateEndpointDefinition(ResourceEndpointDefinition):
 
     @EndpointTask(priority=-101)
     def validate_input(self):
-        """Checks whether there are any unexpected resource fields present. If so, raises an error and returns the unexpected fields."""
-        resource = self.resource
+        """Checks whether there are any unexpected resource fields present. If so,
+        raises an error and returns the unexpected fields.
+        """
         expected_fields = set(
             list(field.name for field in self.get_resource_fields())
             + list(field.name for field in self.get_request_fields())
