@@ -45,10 +45,10 @@ _DESERIALIZERS = collections.defaultdict(
 class HttpResponseServerError(HttpResponse):
     def __init__(self, *args, **kwargs):
         self.error = kwargs.pop("error", None)
-        super(HttpResponseServerError, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
-class Resource(object):
+class Resource:
     """
     Resource. Create one for your URL mappings, just
     like you would with Django. Takes one argument,
@@ -194,8 +194,9 @@ class Resource(object):
         # XXX: this might be a little weird as it'll contain information about the last executed authenticator
         return actor, anonymous, error
 
-    @vary_on_headers("Authorization")
-    def __call__(self, request, *args, **kwargs):
+    # TODO: make this method less complex and remove the `noqa`
+    @vary_on_headers("Authorization")  # noqa: C901
+    def __call__(self, request, *args, **kwargs):  # noqa: C901
         """
         NB: Sends a `Vary` header so we don't cache requests
         that are different (OAuth stuff in `Authorization` header.)
@@ -254,7 +255,7 @@ class Resource(object):
         request = _DESERIALIZERS[request.content_type](self.cleanup_request(request))
 
         try:
-            data = request.POST if request.method == "POST" else request.GET
+            _ = request.POST if request.method == "POST" else request.GET
             status_code, result = meth(request, *args, **kwargs)
         except Exception as e:
             status_code = http.client.BAD_REQUEST
